import { CacheKey } from 'CacheKeys';
declare type Decorator = (target: unknown, propertyKey: string) => void;
/**
 * Maps the property to the return value of `getter`, caching the value in
 * heap (persists across ticks but NOT global resets).
 *
 * @param getter Given the current instance, calculates the property value.
 *               If the getter returns undefined, the cached value will be
 *               used instead.
 */
export declare const heapCacheGetter: (getter: (instance: any) => unknown, invalidateCache?: ((data: any) => boolean) | undefined) => Decorator;
/**
 * Maps the property to the return value of `getter`, caching the value in
 * Memory (persists across ticks AND global resets.)
 *
 * @param getter Given the current instance, calculates the property value.
 *               If the getter returns undefined, the cached value will be
 *               used instead.
 */
export declare const memoryCacheGetter: (key: CacheKey, getter: (instance: any) => unknown, rehydrater?: ((data: any) => unknown) | undefined, invalidateCache?: ((data: any) => boolean) | undefined) => Decorator;
export {};
