'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var defaultRehydrater = function (d) { return d; };
var asRoomPosition = function (pos) {
    if (!pos)
        return;
    return new RoomPosition(pos.x, pos.y, pos.roomName);
};

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var MemoryCache = {
    get: function (key, propertyKey) { var _a, _b; return (_b = (_a = Memory.cache) === null || _a === void 0 ? void 0 : _a[key]) === null || _b === void 0 ? void 0 : _b[propertyKey]; },
    set: function (key, propertyKey, newValue) {
        var _a, _b;
        var _c;
        (_a = Memory.cache) !== null && _a !== void 0 ? _a : (Memory.cache = {});
        (_b = (_c = Memory.cache)[key]) !== null && _b !== void 0 ? _b : (_c[key] = {});
        Memory.cache[key][propertyKey] = newValue;
    }
};
var cache = new WeakMap();
var HeapCache = {
    get: function (key, propertyKey) {
        var _a;
        return ((_a = cache.get(key)) !== null && _a !== void 0 ? _a : {})[propertyKey];
    },
    set: function (key, propertyKey, newValue) {
        var _a;
        var _b;
        cache.set(key, __assign(__assign({}, ((_b = cache.get(key)) !== null && _b !== void 0 ? _b : {})), (_a = {}, _a[propertyKey] = newValue, _a)));
    }
};

/**
 * Caches the property value in Memory.
 *
 * A function should be specified to generate a unique string
 * key for this instance. For example:
 *
 * ```
 * class CachedContainer {
 *   constructor(public id: Id<StructureContainer>) {}
 *
 *   @memoryCache((i: CachedContainer) => i.id)
 *   public structureType?: StructureConstant
 * }
 * ```
 *
 * Some helper key functions are provided such as `keyById`
 * (equivalent to the above).
 */
var memoryCache = function (key, rehydrater) {
    if (rehydrater === void 0) { rehydrater = defaultRehydrater; }
    return function (target, propertyKey) {
        Object.defineProperty(target, propertyKey, {
            get: function () {
                return rehydrater(MemoryCache.get(key(this), propertyKey));
            },
            set: function (newValue) {
                return MemoryCache.set(key(this), propertyKey, newValue);
            },
            enumerable: true
        });
    };
};

/**
 * Keys the Memory cache to `instance.id` (should be globally unique)
 *
 * @param i Parent class instance
 */
var keyById = function (i) { return i.id; };
/**
 * Keys the Memory cache to `instance.name` (should be globally unique)
 *
 * @param i Parent class instance
 */
var keyByName = function (i) { return i.name; };

var cacheGetter = function (cacheMethod, key, getter, rehydrater, invalidateCache) {
    // Decorator scope requires us to maintain a cache
    // for each instance. WeakMap will drop any instances
    // once they have no other references to avoid memory
    // leaks. The mapped cache object itself will have a
    // key for each cached property.
    if (rehydrater === void 0) { rehydrater = function (d) { return d; }; }
    if (invalidateCache === void 0) { invalidateCache = function () { return false; }; }
    return function (target, propertyKey) {
        // We assign getter and setter to the property on the
        // class when it is defined. When the class is instantiated,
        // `this` begins to reference the instance rather than
        // the source class, so the cache indexes by instance
        Object.defineProperty(target, propertyKey, {
            get: function () {
                var value = getter(this);
                if (value === undefined && !invalidateCache(value)) {
                    value = cacheMethod.get(key(this), propertyKey);
                }
                cacheMethod.set(key(this), propertyKey, value);
                return rehydrater(value);
            },
            set: function () {
                // Set from game object only
            },
            enumerable: true
        });
    };
};
/**
 * Keys the Heap cache to the instance reference (used internally)
 *
 * @param i Parent class instance
 */
var keyByInstance = function (i) { return i; };
/**
 * Maps the property to the return value of `getter`, caching the value in
 * heap (persists across ticks but NOT global resets).
 *
 * @param getter Given the current instance, calculates the property value.
 *               If the getter returns undefined, the cached value will be
 *               used instead.
 */
var heapCacheGetter = function (getter, invalidateCache) {
    return cacheGetter(HeapCache, keyByInstance, getter, undefined, invalidateCache);
};
/**
 * Maps the property to the return value of `getter`, caching the value in
 * Memory (persists across ticks AND global resets.)
 *
 * @param getter Given the current instance, calculates the property value.
 *               If the getter returns undefined, the cached value will be
 *               used instead.
 */
var memoryCacheGetter = function (key, getter, rehydrater, invalidateCache) {
    return cacheGetter(MemoryCache, key, getter, rehydrater, invalidateCache);
};

exports.HeapCache = HeapCache;
exports.MemoryCache = MemoryCache;
exports.asRoomPosition = asRoomPosition;
exports.defaultRehydrater = defaultRehydrater;
exports.heapCacheGetter = heapCacheGetter;
exports.keyById = keyById;
exports.keyByName = keyByName;
exports.memoryCache = memoryCache;
exports.memoryCacheGetter = memoryCacheGetter;
//# sourceMappingURL=main.js.map
